!
! Copyright (c) 2015 Edward D. Zaron, Portland, Oregon, USA
!
! Permission is hereby granted, free of charge, to any person obtaining a copy of this
! software and associated documentation files (the "Software"), to deal in the Software
! without restriction, including without limitation the rights to use, copy, modify, merge,
! publish, distribute, sublicense, and/or sell copies of the Software, and to permit
! persons to whom the Software is furnished to do so, subject to the following conditions:
!
! The above copyright notice and this permission notice shall be included in all copies or
! substantial portions of the Software.
!
! Except as contained in this notice, the name(s) of the above copyright holders shall not
! be used in advertising or otherwise to promote the sale, use or other dealings in this
! Software without prior written authorization.
!
! THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
! INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
! PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
! FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
! OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
! DEALINGS IN THE SOFTWARE.
!

module kinds
  implicit none
  
  integer(kind=4), public, parameter :: int_kind = 4
  integer(kind=4), public, parameter :: real_kind = 4
  integer(kind=4), public, parameter :: ptr_kind = 8
  
  integer(kind=4), public, parameter :: STRLEN = 4096
  integer(kind=4), public, parameter :: XSTRLEN = 4096
  
end module kinds
